//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "FormMain.h"
#include "FormAbout.h"
#include "FormSettings.h"
#include "FormLog.h"
#include "Log.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TfrmMain *frmMain;
//---------------------------------------------------------------------------
__fastcall TfrmMain::TfrmMain(TComponent* Owner)
	: TForm(Owner), lpt(INVALID_HANDLE_VALUE)
{
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::FormCreate(TObject *Sender)
{
	AnsiString asConfigFile = ChangeFileExt( Application->ExeName, ".ini" );
	appSettings.Read(asConfigFile);
	if (this->BorderStyle != bsSingle)
	{
		this->Width = appSettings.frmMain.iWidth;
		this->Height = appSettings.frmMain.iHeight;
	}
	this->Top = appSettings.frmMain.iPosY;
	this->Left = appSettings.frmMain.iPosX;
	if (appSettings.frmMain.bAlwaysOnTop)
		this->FormStyle = fsStayOnTop;
	else
		this->FormStyle = fsNormal;
	if (appSettings.frmMain.bWindowMaximized)
		this->WindowState = wsMaximized;
	if (appSettings.Logging.bLogToFile)
		CLog::Instance()->SetFile(ChangeFileExt(Application->ExeName, ".log").c_str());
	else
		CLog::Instance()->SetFile("");
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::FormCloseQuery(TObject *Sender, bool &CanClose)
{
	AnsiString asConfigFile = ChangeFileExt( Application->ExeName, ".ini" );
	appSettings.frmMain.bWindowMaximized = (this->WindowState == wsMaximized);
	if (!appSettings.frmMain.bWindowMaximized)
	{
		// these values are meaningless is wnd is maximized
		appSettings.frmMain.iWidth = this->Width;
		appSettings.frmMain.iHeight = this->Height;
		appSettings.frmMain.iPosY = this->Top;
		appSettings.frmMain.iPosX = this->Left;
	}
	appSettings.Write(asConfigFile);

	CanClose = true;	
}
//---------------------------------------------------------------------------
void __fastcall TfrmMain::actShowAboutExecute(TObject *Sender)
{
	frmAbout->ShowModal();	
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::actShowSettingsExecute(TObject *Sender)
{
	frmSettings->appSettings = &appSettings;
	frmSettings->ShowModal();
	if (appSettings.frmMain.bAlwaysOnTop)
		this->FormStyle = fsStayOnTop;
	else
		this->FormStyle = fsNormal;
	if (appSettings.Logging.bLogToFile)
		CLog::Instance()->SetFile(ChangeFileExt(Application->ExeName, ".log").c_str());
	else
		CLog::Instance()->SetFile("");
	frmLog->SetBufferSize(appSettings.Logging.iMaxUiLogLines);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::FormShow(TObject *Sender)
{
    static bool once = false;
    if (!once)
    {
		once = true;
		frmLog->SetBufferSize(appSettings.Logging.iMaxUiLogLines);		
		CLog::Instance()->SetLevel(E_LOG_TRACE);
		CLog::Instance()->callbackLog = frmLog->OnLog;
	}
	LOG("Application started");
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::FormDestroy(TObject *Sender)
{
	CLog::Instance()->Destroy();	
	if (lpt != INVALID_HANDLE_VALUE)
		CloseHandle(lpt);
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::actShowLogExecute(TObject *Sender)
{
	if (!frmLog->Visible)
		frmLog->Show();
	frmLog->BringToFront();
}
//---------------------------------------------------------------------------



void __fastcall TfrmMain::btnOpenClick(TObject *Sender)
{
	if (lpt != INVALID_HANDLE_VALUE)
		CloseHandle(lpt);
	lpt = CreateFile(cbLpt->Text.c_str(),
		GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, 0);
	if (lpt == INVALID_HANDLE_VALUE)
	{
		ReportError("Could not open LPT port");
		return;
	}

	/** \note
	http://www.codeguru.com/forum/showthread.php?t=231214
	The parallelTimeouts.WriteTotalTimeoutConstant variable has a minimum value of 2000 (2 seconds) on WinXP.
	Default value is set to 300000 (5 mins).
	*/
	COMMTIMEOUTS parallelTimeouts;
	parallelTimeouts.ReadIntervalTimeout = MAXDWORD;
	parallelTimeouts.ReadTotalTimeoutMultiplier = 3000;
	parallelTimeouts.ReadTotalTimeoutConstant = 3000;
	parallelTimeouts.WriteTotalTimeoutMultiplier = 3000;
	parallelTimeouts.WriteTotalTimeoutConstant = 3000;

	if (!SetCommTimeouts(lpt, &parallelTimeouts))
	{
		 ReportError("SetCommTimeouts failed");
		 return;
	}
	SetStatus("%s opened", cbLpt->Text.c_str());
}
//---------------------------------------------------------------------------

void TfrmMain::ReportError(AnsiString caption)
{
	LPVOID lpMsgBuf;
	DWORD dw = GetLastError();
	FormatMessage(
		FORMAT_MESSAGE_ALLOCATE_BUFFER |
		FORMAT_MESSAGE_FROM_SYSTEM |
		FORMAT_MESSAGE_IGNORE_INSERTS,
		NULL,
		dw,
		MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
		(LPTSTR) &lpMsgBuf,
		0, NULL );
	MessageBox(this->Handle, (LPCTSTR)lpMsgBuf, caption.c_str(), MB_ICONEXCLAMATION);
	LocalFree(lpMsgBuf);
}

void TfrmMain::SetStatus(char *lpData, ...)
{
	AnsiString text;
	va_list ap;
	va_start(ap, lpData);
	text.vprintf(lpData, ap);
	va_end(ap);
	StatusBar->SimpleText = text;
}

void __fastcall TfrmMain::btnCloseClick(TObject *Sender)
{
	if (lpt != INVALID_HANDLE_VALUE)
		CloseHandle(lpt);
	SetStatus("Port closed");
	lpt = INVALID_HANDLE_VALUE;
}
//---------------------------------------------------------------------------

void __fastcall TfrmMain::btnWriteClick(TObject *Sender)
{
	if (lpt == INVALID_HANDLE_VALUE)
		return;
	unsigned char byte = 0;
	for (int i=0; i<8; i++)
	{
		if (chblistValue->Checked[i])
			byte |= (unsigned char)(1 << i);
	}
	DWORD dwBytesWritten;
	if (!WriteFile(lpt, &byte, 1, &dwBytesWritten, NULL))
		ReportError("Write error");
}
//---------------------------------------------------------------------------

